@@ -3,7 +3,7 @@
 
 ##############################################################
 ##   smscon-editor  -  Configuration UI for the SMSCON      ##
-# $Id: smsconEditorMain.py 84 2011-12-24 02:21:55Z saturn $
+# $Id: smsconEditorMain.py 108 2011-12-28 18:46:50Z saturn $
 ##############################################################
 # Copyright (c) 2010-2011 Christos Zamantzas
 # Licenced under GPLv2
@@ -22,7 +22,7 @@
 ## GNU General Public License for more details.
 ##
 
-Version = '0.9.6-1'
+Version = '0.9.8-1'
 
 import os
 import sys
@@ -66,6 +66,8 @@ FatalHelp       = 'Please contact SMSCON support team for a solution.'
 
 editConf        = UserConfiguration(Filename = os.path.join(TempPath, smscon.ConfigFile))
 
+areFeedbackDetailsDesired = False
+
 def commonPrefix(*args):
     """@return string The longest same string that all passed strings are starting with"""
     for idx in range(min(map(len, args))):
@@ -98,9 +100,16 @@ def showLogRecord(logRecord):
 
     msg = "%s: %s" % (logRecord.levelname, logRecord.msg)
     if logRecord.levelname == 'ERROR':
-        showMessage(msg)
+       showMessage(msg)
+    elif logRecord.levelname == 'WARNING':   
+       showMessage(msg)
+    elif logRecord.levelname == 'CRITICAL':   
+       showMessage(msg)
     else:
-        showQuickMessage(msg)
+       if areFeedbackDetailsDesired:
+          showQuickMessage(msg)
+       else:
+          pass
 
 LOGGER = smscon.StartLogging('EDITOR', withConsole = False, withEmitterFxn = showLogRecord)
 
@@ -117,17 +126,17 @@ def IsPasswordValid(password):  ### FIXME: what should be the behaviour if passw
     '''
 
     try:
-        filesize = os.path.getsize(PassFile)
+       filesize = os.path.getsize(PassFile)
     except:
-        message = 'Missing password file!\n%s' % FatalHelp
-        showMessage(message)
-        #doUpdatePassword()
-        sys.exit(1)
+       message = 'Missing password file!\n%s' % FatalHelp
+       showMessage(message)
+       #doUpdatePassword()
+       sys.exit(1)
 
     if filesize == 0:
-        message = 'Empty password file!\n%s' % FatalHelp
-        showMessage(message)
-        sys.exit(1)
+       message = 'Empty password file!\n%s' % FatalHelp
+       showMessage(message)
+       sys.exit(1)
 
     return os.popen('echo %s|md5sum -c "%s"' % (shellArg(password), PassFile)).read() == '-: OK\n'
 
@@ -139,45 +148,45 @@ def CopyMainToEditConfig():
 
     def tryCopy():
         
-        global editConf
-        
-        if os.WEXITSTATUS(os.system('sudo smscon -export! "%s"' % editConf.Filename)) == 0:
-            return True
-        msg1 = 'Loading active settings failed.'
+       global editConf
+       
+       if os.WEXITSTATUS(os.system('sudo smscon -export! "%s"' % editConf.Filename)) == 0:
+          return True
+       msg1 = 'Loading active settings failed.'
 
-        if os.WEXITSTATUS(os.system('sudo smscon -init')) == 0:
-            msg2 = '\nActivated initial settings.'
-        else:
-            msg2 = '\nActivating initial settings failed.'
+       if os.WEXITSTATUS(os.system('sudo smscon -init')) == 0:
+          msg2 = '\nActivated initial settings.'
+       else:
+          msg2 = '\nActivating initial settings failed.'
 
-        if not IsFileReadable(editConf.Filename):   # os.path.isfile() checks existence only, not accessibility
-            showMessage(msg1+msg2)
-            return False
-        # Try to restore recent pre-edit settings.
-        return EditConfigAndApply(None,
-            startMessage    = None,
-            successMessage  = msg1 + '\nActivated recent edit settings.',
-            errorMessage    = msg1 + '\nFailed to activate recent edit settings.' + msg2,
-        )
+       if not IsFileReadable(editConf.Filename):   # os.path.isfile() checks existence only, not accessibility
+          showMessage(msg1+msg2)
+          return False
+       # Try to restore recent pre-edit settings.
+       return EditConfigAndApply(None,
+          startMessage    = None,
+          successMessage  = msg1 + '\nActivated recent edit settings.',
+          errorMessage    = msg1 + '\nFailed to activate recent edit settings.' + msg2,
+          )
 
     ok = tryCopy() or tryCopy() # 2nd call may succeed if 1st could activate some of recent edit settings.
 
     global editConf
     if not editConf.load():
-        if not editConf.needsUpgrade():
-            message = 'Failed to read SMSCON settings.\n%s' % FatalHelp
-            showMessage(message)
-            sys.exit(1)
-        if not editConf.upgrade():
-            message = 'Failed to upgrade SMSCON settings.\n%s' % FatalHelp
-            showMessage(message)
-            sys.exit(1)
+       if not editConf.needsUpgrade():
+          message = 'Failed to read SMSCON settings.\n%s' % FatalHelp
+          showMessage(message)
+          sys.exit(1)
+       if not editConf.upgrade():
+          message = 'Failed to upgrade SMSCON settings.\n%s' % FatalHelp
+          showMessage(message)
+          sys.exit(1)
     if ok:
-        message = 'Successfully loaded SMSCON settings.'
-        showQuickMessage(message)
+       message = 'Successfully loaded SMSCON settings.'
+       showQuickMessage(message)
     else:
-        message = 'Enforcedly loaded SMSCON settings.\nPlease check them all. Some seem to be invalid.'
-        showMessage(message)
+       message = 'Enforcedly loaded SMSCON settings.\nPlease check them all. Some seem to be invalid.'
+       showMessage(message)
     return True
 
 def EditConfigAndApply(newConf, successMessage = None, errorMessage = None, startMessage = 'Updating. Please wait...'):
@@ -188,18 +197,22 @@ def EditConfigAndApply(newConf, successMessage = None, errorMessage = None, star
     global editConf
     ok = True
     if newConf:
-        if startMessage:
-            showQuickMessage(startMessage)
-        for k in newConf:
-            ok = editConf.update(k, newConf[k]) and ok
+       if startMessage:
+          if areFeedbackDetailsDesired:
+             showQuickMessage(startMessage)
+       for k in newConf:
+          ok = editConf.update(k, newConf[k]) and ok
 
     ok = ok and os.WEXITSTATUS(os.system('sudo smscon -import "%s"' % editConf.Filename)) == 0
     if ok:
-        if successMessage:
-            showMessage(successMessage)
+       if areFeedbackDetailsDesired:
+          if successMessage:
+             showMessage(successMessage)
+          else:
+             pass
     else:
-        if errorMessage:
-            showMessage(errorMessage)
+       if errorMessage:
+          showMessage(errorMessage)
     return ok
 
 class smsconEditorStartup(QtGui.QDialog):
@@ -228,52 +241,50 @@ class smsconEditorStartup(QtGui.QDialog):
 class smsconEditorPasswordRequest(QtGui.QDialog): ### FIXME: add 3 re-tries for wrong password.
     def __init__(self, parent=None):
 
-        ##Build parent user interface
-        QtGui.QDialog.__init__(self, parent)
-        self.ui = Ui_smsconEditorAskPassUI()
-        self.ui.setupUi(self)
-        self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
-        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
-        #self.showFullScreen()
-        self.isInitialPasswordEntered = None
+       ##Build parent user interface
+       QtGui.QDialog.__init__(self, parent)
+       self.ui = Ui_smsconEditorAskPassUI()
+       self.ui.setupUi(self)
+       self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
+       self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
+       #self.showFullScreen()
+       self.isInitialPasswordEntered = None
 
-        ##Connect the button
-        QtCore.QObject.connect(self.ui.btnEnterPassword, QtCore.SIGNAL('clicked()'), self.doEnterPassword)
+       ##Connect the button
+       QtCore.QObject.connect(self.ui.btnEnterPassword, QtCore.SIGNAL('clicked()'), self.doEnterPassword)
 
     def doEnterPassword(self):
 
-        self.close()
+       self.close()
 
     ##Close app if window closes without a valid password
     def closeEvent(self, event):
-        """
-        Check the entered password.
-        This dialog MUST NOT RETURN TO THE APPLICATION in case the entered password is wrong
-        """
+       """
+       Check the entered password.
+       This dialog MUST NOT RETURN TO THE APPLICATION in case the entered password is wrong
+       """
 
-        event.accept()
-        try:
-            EnteredPassword = self.ui.linePassword.text()
-            if IsPasswordValid(EnteredPassword):
-                self.isInitialPasswordEntered = EnteredPassword == InitialPassword 
-                return
-        except:
-            pass
+       event.accept()
+       try:
+          EnteredPassword = self.ui.linePassword.text()
+          if IsPasswordValid(EnteredPassword):
+             self.isInitialPasswordEntered = EnteredPassword == InitialPassword 
+             return
+       except:
+          pass
 
-        if len(EnteredPassword) > 0:
-            Hostname = os.uname()[1]
-            message = 'N900[%s]\nSMSCON Editor: wrong password (length %d) entered.' % (Hostname, len(EnteredPassword))
-            os.system('sudo smscon -sendsms MASTERNUMBER %s&' % shellArg(message))
-        message = 'Incorrect password. Exiting.'
-        showQuickMessage(message)
-        sys.exit(1)
+       if len(EnteredPassword) > 0:
+          Hostname = os.uname()[1]
+          message = 'N900[%s]\nSMSCON Editor: wrong password (length %d) entered.' % (Hostname, len(EnteredPassword))
+          os.system('sudo smscon -sendsms MASTERNUMBER %s&' % shellArg(message))
+       message = 'Incorrect password. Exiting.'
+       showQuickMessage(message)
+       sys.exit(1)
 
 class smsconEditorMainWindow(QtGui.QMainWindow):
     def __init__(self, parent=None):
 
        global editConf
-       global areFeedbackDetailsDesired
-       areFeedbackDetailsDesired = True
 
        self.isUsersFirstTimeCall = not os.path.isfile(editConf.Filename)
        if self.isUsersFirstTimeCall:
@@ -291,10 +302,10 @@ class smsconEditorMainWindow(QtGui.QMainWindow):
         
        if self.isUsersFirstTimeCall:
           message =   'Seems this is your first time here. You now should\n' \
-                      '\n1. Edit settings to your needs.' \
-                      '\n2. Check operation on "Test" tab.' \
+                      '\n1. Edit all settings to your needs.' \
+                      '\n2. Check operation on the "Test" window.' \
                       '\n3. Enable SMSCON on device boot and' \
-                      '\n4. Start SMSCON operation on "Init" tab.'
+                      '\n4. Start SMSCON operation.'
           if passwordDialog.isInitialPasswordEntered:
              message += '\n\nAnd please go to "Change Password" and do so!'
           showMessage(message)
@@ -341,12 +352,12 @@ class smsconEditorMainWindow(QtGui.QMainWindow):
        self.SetSystemValues()
 
     def closeEvent(self, event):
-        """Cleanup on close."""
-        event.accept()
-        if self.isSmsconTestActive:
-            message = 'Test mode was active.\nSwitching to normal operation...'
-            showQuickMessage(message)
-            self.doRestart();
+       """Cleanup on close."""
+       event.accept()
+       if self.isSmsconTestActive:
+          message = 'Test mode was active.\nSwitching to normal operation...'
+          showQuickMessage(message)
+       self.doRestart();
 
     def SetSystemValues(self):
        '''Set GUI values from the current operation'''
@@ -354,7 +365,7 @@ class smsconEditorMainWindow(QtGui.QMainWindow):
        DAEMONSTATE = os.popen('sudo smscon -status').read().strip('\n') 
        BOOTSTATE = IsFileReadable('/etc/event.d/smscon_boot')
        
-       if DAEMONSTATE == 'Daemon smscon_daemon is running':
+       if DAEMONSTATE.endswith('running'):
           self.ui.btnStop.setEnabled(True)
           self.ui.btnStop.setChecked(True)
           self.ui.btnStart.setEnabled(False)
@@ -433,8 +444,7 @@ class smsconEditorMainWindow(QtGui.QMainWindow):
        passwordChangeDialog = smsconEditorPasswordChange()
        passwordChangeDialog.exec_()
 
-## Initialise SMSCON
-class smsconEditorInit(QtGui.QMainWindow):
+class smsconEditorInit(QtGui.QMainWindow):     ## Initialise SMSCON
 
     def __init__(self, parent=None):
 
@@ -448,10 +458,10 @@ class smsconEditorInit(QtGui.QMainWindow):
 
        ##Connect the GUI Buttons with actions
           ##Create Settings
-       QtCore.QObject.connect(self.ui.btnInit, QtCore.SIGNAL('clicked()'), self.doInit)
-       QtCore.QObject.connect(self.ui.btnReset, QtCore.SIGNAL('clicked()'), self.doReset)
-       QtCore.QObject.connect(self.ui.btnDelIMSI, QtCore.SIGNAL('clicked()'), self.doDelIMSI)
-       QtCore.QObject.connect(self.ui.btnAddIMSI, QtCore.SIGNAL('clicked()'), self.doAddIMSI)
+       QtCore.QObject.connect(self.ui.btnInit,     QtCore.SIGNAL('clicked()'), self.doInit)
+       QtCore.QObject.connect(self.ui.btnReset,    QtCore.SIGNAL('clicked()'), self.doReset)
+       QtCore.QObject.connect(self.ui.btnDelIMSI,  QtCore.SIGNAL('clicked()'), self.doDelIMSI)
+       QtCore.QObject.connect(self.ui.btnAddIMSI,  QtCore.SIGNAL('clicked()'), self.doAddIMSI)
 
     def doInit(self):
         os.system('sudo smscon -init!')
@@ -487,8 +497,7 @@ class smsconEditorInit(QtGui.QMainWindow):
        message = os.popen('sudo smscon -add imsi 2>&1').read()
        showMessage(message)
      
-## General Settings
-class smsconEditorGeneral(QtGui.QMainWindow):
+class smsconEditorGeneral(QtGui.QMainWindow):  ## General Settings
 
     def __init__(self, parent=None):
 
@@ -500,21 +509,19 @@ class smsconEditorGeneral(QtGui.QMainWindow):
        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
        self.show()       
 
-       ##Connect the GUI Buttons with actions
-       QtCore.QObject.connect(self.ui.btnCreateConfigGeneral, QtCore.SIGNAL('clicked()'), self.doCreateConfigGeneral)
-
        ##Set UI values from editConfig values
        global editConf
        
-       self.ui.linePhone         .setText(editConf['MASTERNUMBER'])
-       self.ui.lineGPSTimeout    .setText(editConf['GPSTIMEOUT'])  
-       self.ui.lineGPSPolling    .setText(editConf['GPSPOLLING'])
+       self.ui.linePhoneMaster   .setText(editConf['MASTERNUMBER'])
+       self.ui.linePhoneSender   .setText(editConf['SENDERNUMBER'])
+       self.ui.spinBoxGPSTimeout .setValue(int(editConf['GPSTIMEOUT']))  
+       self.ui.spinBoxGPSPolling .setValue(int(editConf['GPSPOLLING']))
 
        for idx, seconds in enumerate((10, 20, 30, 60, 120)):
           if int(editConf['GPSINTERVAL']) <= seconds:
              break;
-       self.ui.comboBoxGPSInterval.setCurrentIndex(idx)  # idx is 4 if GPSINTERVAL > 120 
-       self.ui.comboBoxGPSSend    .setCurrentIndex(['sms', 'email', 'both'].index(editConf['GPSSEND'])) 
+       self.ui.comboBoxGPSInterval   .setCurrentIndex(idx)  # idx is 4 if GPSINTERVAL > 120 
+       self.ui.comboBoxGPSSend       .setCurrentIndex(['sms', 'email', 'both'].index(editConf['GPSSEND'])) 
 
        self.ui.comboBoxReplyMethod   .setCurrentIndex(['sms', 'email', 'both', 'none'].index(editConf['MESSAGESEND'])) 
        self.ui.checkBoxCommandReply  .setEnabled(editConf['MESSAGESEND'] != 'none')
@@ -529,45 +536,51 @@ class smsconEditorGeneral(QtGui.QMainWindow):
        self.ui.checkBoxSilence       .setChecked(configTrue(editConf['SILENCEDEVICE']))
            
        self.ui.checkBoxEnableRetry   .setChecked(configTrue(editConf['ENABLERESEND']))
-       self.ui.lineReplyWait         .setEnabled(configTrue(editConf['ENABLERESEND']))
-       self.ui.lineReplyWait         .setText(editConf['RESENDTIME'])
-       self.ui.lineRetryAttempts     .setEnabled(configTrue(editConf['ENABLERESEND']))
-       self.ui.lineRetryAttempts     .setText(editConf['RESENDMAXTRY'])
+       self.ui.doubleSpinBoxReplyWait.setEnabled(configTrue(editConf['ENABLERESEND']))
+       self.ui.doubleSpinBoxReplyWait.setValue(float(editConf['RESENDTIME']))
+       self.ui.spinBoxRetryAttempts  .setEnabled(configTrue(editConf['ENABLERESEND']))
+       self.ui.spinBoxRetryAttempts  .setValue(int(editConf['RESENDMAXTRY']))
        
+    def closeEvent(self, event):
+       """Store settings on close."""
+       event.accept()
+
+       self.doCreateConfigGeneral()
+       # self.doRestart();
         
     def doCreateConfigGeneral(self):
 
-        newConf = {}
-        newConf["MASTERNUMBER"]   = self.ui.linePhone.text()
-        newConf["GPSTIMEOUT"]     = self.ui.lineGPSTimeout.text()
-        newConf["GPSPOLLING"]     = self.ui.lineGPSPolling.text()
-        newConf["GPSINTERVAL"]    = self.ui.comboBoxGPSInterval.currentText()
-        newConf["GPSSEND"]        = self.ui.comboBoxGPSSend.currentText()
+       newConf = {}
+       newConf["MASTERNUMBER"]   = self.ui.linePhoneMaster.text()
 
-        newConf["MESSAGESEND"]    = self.ui.comboBoxReplyMethod.currentText()
-        newConf["RESENDTIME"]     = self.ui.lineReplyWait.text()
-        newConf["RESENDMAXTRY"]   = self.ui.lineRetryAttempts.text()
+       newConf["GPSTIMEOUT"]     = str(self.ui.spinBoxGPSTimeout.value())
+       newConf["GPSPOLLING"]     = str(self.ui.spinBoxGPSPolling.value())
+       newConf["GPSINTERVAL"]    = self.ui.comboBoxGPSInterval.currentText()
+       newConf["GPSSEND"]        = self.ui.comboBoxGPSSend.currentText()
 
-        newConf["ENABLERESEND"]   = configYesNo(self.ui.checkBoxEnableRetry.isChecked())
-        newConf["COMMANDREPLY"]   = configYesNo(self.ui.checkBoxCommandReply.isChecked())
-        newConf["AUTODEVICELOCK"] = configYesNo(self.ui.checkBoxDeviceLock.isChecked())
-        newConf["AUTOBATREPORT"]  = configYesNo(self.ui.checkBoxLowBattery.isChecked())
-        newConf["SIMUNLOCK"]      = configYesNo(self.ui.checkBoxNewSIMUnlock.isChecked())
-        newConf["SILENCEDEVICE"]  = configYesNo(self.ui.checkBoxSilence.isChecked())
-        newConf["KEYBOARDDETECT"] = configYesNo(self.ui.checkBoxKeyboardDetect.isChecked())
+       newConf["MESSAGESEND"]    = self.ui.comboBoxReplyMethod.currentText()
+       newConf["RESENDTIME"]     = str(self.ui.doubleSpinBoxReplyWait.value())
+       newConf["RESENDMAXTRY"]   = str(self.ui.spinBoxRetryAttempts.value())
+       newConf["ENABLERESEND"]   = configYesNo(self.ui.checkBoxEnableRetry.isChecked())
 
-        newConf['SENDERNUMBER']  = newConf["MASTERNUMBER"]
+       newConf["COMMANDREPLY"]   = configYesNo(self.ui.checkBoxCommandReply.isChecked())
+       newConf["AUTODEVICELOCK"] = configYesNo(self.ui.checkBoxDeviceLock.isChecked())
+       newConf["AUTOBATREPORT"]  = configYesNo(self.ui.checkBoxLowBattery.isChecked())
+       newConf["SIMUNLOCK"]      = configYesNo(self.ui.checkBoxNewSIMUnlock.isChecked())
+       newConf["SILENCEDEVICE"]  = configYesNo(self.ui.checkBoxSilence.isChecked())
+       newConf["KEYBOARDDETECT"] = configYesNo(self.ui.checkBoxKeyboardDetect.isChecked())
 
-        message =   'Settings have been successfully saved.' \
-                    '\n%s remembered as master number.' % newConf["MASTERNUMBER"]
-        if areFeedbackDetailsDesired:
-            message +=  '\nAn SMS will be send to it if the SIM ever gets replaced.' \
-                    '\n\nControlling SMS can be send from master or any other phone. ' \
-                    'If numbers differ, SMS notifications will be send to both.'
-        EditConfigAndApply(newConf, message)
+       newConf['SENDERNUMBER']   = newConf["MASTERNUMBER"]
 
-## Email Settings    
-class smsconEditorEmail(QtGui.QMainWindow):
+       message =   'Settings have been successfully saved.' \
+                   '\n%s remembered as master number.' % newConf["MASTERNUMBER"]
+       if areFeedbackDetailsDesired:
+          message +=  '\nAn SMS will be send to it if the SIM ever gets replaced.' \
+                      '\n\nControlling SMS can be send from master or any other phone. ' \
+                      'If numbers differ, SMS notifications will be send to both.'
+       EditConfigAndApply(newConf, message)
+
+class smsconEditorEmail(QtGui.QMainWindow):    ## Email Settings 
 
     def __init__(self, parent=None):
 
@@ -579,9 +592,6 @@ class smsconEditorEmail(QtGui.QMainWindow):
        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
        self.show()       
 
-       ##Connect the GUI Buttons with actions
-       QtCore.QObject.connect(self.ui.btnCreateConfigEmail, QtCore.SIGNAL('clicked()'), self.doCreateConfigEmail)
-
        ##Set UI values from editConfig values
        global editConf
        
@@ -591,6 +601,13 @@ class smsconEditorEmail(QtGui.QMainWindow):
        self.ui.lineEmailFrom     .setText(editConf['EMAILFROM'])
        self.ui.lineEmailServer   .setText(editConf['MAILSERVER'])
        self.ui.lineEmailPort     .setText(editConf['MAILPORT'])
+
+    def closeEvent(self, event):
+       """Store settings on close."""
+       event.accept()
+
+       self.doCreateConfigEmail()
+       # self.doRestart();
        
     def doCreateConfigEmail(self):
 
@@ -604,16 +621,15 @@ class smsconEditorEmail(QtGui.QMainWindow):
         
         message =   'The mail settings have been successfully saved.'
         if areFeedbackDetailsDesired:
-            message +=  '\nRecipient address: %s' % newConf["EMAILADDRESS"] + \
-                    '\n' \
-                    '\nSender address: %s' % newConf["EMAILFROM"] + \
-                    '\nUser name: %s' % newConf["USER"] + \
-                    '\nPassword: %s' % (newConf["PASSWORD"] and '******') + \
-                    '\nMail server: %s - Port: %s' % (newConf["MAILSERVER"], newConf["MAILPORT"])
+            message +=   '\nRecipient address: %s' % newConf["EMAILADDRESS"] + \
+                         '\n' \
+                         '\nSender address: %s' % newConf["EMAILFROM"] + \
+                         '\nUser name: %s' % newConf["USER"] + \
+                         '\nPassword: %s' % (newConf["PASSWORD"] and '******') + \
+                         '\nMail server: %s - Port: %s' % (newConf["MAILSERVER"], newConf["MAILPORT"])
         EditConfigAndApply(newConf, message)
         
-## Commands Renaming    
-class smsconEditorCommands(QtGui.QMainWindow):
+class smsconEditorCommands(QtGui.QMainWindow): ## Commands Renaming
 
     def __init__(self, parent=None):
 
@@ -625,21 +641,18 @@ class smsconEditorCommands(QtGui.QMainWindow):
        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
        self.show()       
 
-       ##Connect the GUI Buttons with actions
-       QtCore.QObject.connect(self.ui.btnCreateConfigCommands, QtCore.SIGNAL('clicked()'), self.doCreateConfigCommands)
-
        ##Set UI values from editConfig values
        global editConf
-       # Make a copy of command strings because we may change them for displaying
-       # in case we detect old style of command prefixes.:
+       ##Make a copy of command strings because we may change them for displaying
+       ##in case we detect old style of command prefixes:
        comConf = {}
        for cmd in editConf.getSmsCommandList():
           comConf[cmd] = editConf[cmd]
 
        prefix = editConf['SMS_COMPREFIX']
        if len(prefix) == 0:
-          # Check for old style prefix which was explicitly part of each
-          # command (except for COM_SHELL, where explicit prefix didn't work).
+          ##Check for old style prefix which was explicitly part of each
+          ##command (except for COM_SHELL, where explicit prefix didn't work).
           prefix = commonPrefix(*(comConf[cmd] for cmd in comConf if cmd != 'COM_SHELL' ))
           if len(prefix) > 0:
              prefixRemoved = 0
@@ -659,13 +672,13 @@ class smsconEditorCommands(QtGui.QMainWindow):
              else:
                 prefix = ''
 
-       # Display the prefix and suffix separately ... 
+       ##Display the prefix and suffix separately ... 
        self.ui.linePrefix      .setText(prefix)
        self.ui.lineSuffix      .setText(editConf['SMS_COMSUFFIX']);
         
-       # ... and show all the commands without the prefix (and without the suffix).
+       ##... and show all the commands without the prefix (and without the suffix).
        self.ui.lineCheck       .setText(comConf['COM_CHECK'])
-       #self.ui.lineLog    .setText(comConf["COM_LOG"])        -- TODO: ADD TO GUI (reply the smscon.log)
+       self.ui.lineLog         .setText(comConf["COM_LOG"])        ##reply the smscon.log
        self.ui.lineReboot      .setText(comConf['COM_REBOOT'])
        self.ui.linePowerOff    .setText(comConf['COM_POWEROFF'])
        self.ui.linePower       .setText(comConf['COM_POWER'])
@@ -679,46 +692,52 @@ class smsconEditorCommands(QtGui.QMainWindow):
        self.ui.lineTrackOn     .setText(comConf['COM_TRACKON'])
        self.ui.lineTrackOff    .setText(comConf['COM_TRACKOFF'])
        self.ui.lineScript      .setText(comConf['COM_CUSTOM'])
-       #self.ui.lineScriptLog    .setText(comConf["COM_CUSTOMLOG"])        -- TODO: ADD TO GUI (reply the COM_CUSTOM output)
+       self.ui.lineScriptLog   .setText(comConf["COM_CUSTOMLOG"])  ##reply the COM_CUSTOM output
        self.ui.lineShell       .setText(comConf['COM_SHELL'])
        self.ui.lineAlarm       .setText(comConf['COM_ALARM'])
        self.ui.lineRestart     .setText(comConf['COM_RESTART'])
+
+    def closeEvent(self, event):
+       """Store settings on close."""
+       event.accept()
+
+       self.doCreateConfigCommands()
+       # self.doRestart();
        
     def doCreateConfigCommands(self):
 
-        newConf = {}
-        newConf['SMS_COMPREFIX']    = self.ui.linePrefix.text()
-        newConf['SMS_COMSUFFIX']    = self.ui.lineSuffix.text()
+       newConf = {}
+       newConf['SMS_COMPREFIX']    = self.ui.linePrefix.text()
+       newConf['SMS_COMSUFFIX']    = self.ui.lineSuffix.text()
 
-        newConf["COM_CHECK"]        = self.ui.lineCheck.text()
-        #newConf["COM_LOG"]          = self.ui.lineLog.text()        -- TODO: ADD TO GUI
-        newConf["COM_LOCATION"]     = self.ui.lineLocation.text()
-        newConf["COM_TRACKON"]      = self.ui.lineTrackOn.text()
-        newConf["COM_TRACKOFF"]     = self.ui.lineTrackOff.text()
-        newConf["COM_CAMERA"]       = self.ui.lineCamera.text()
-        newConf["COM_CALL"]         = self.ui.lineCall.text()
-        newConf["COM_LOCK"]         = self.ui.lineLock.text()
-        newConf["COM_UNLOCK"]       = self.ui.lineUnlock.text()
-        newConf["COM_REMOTEON"]     = self.ui.lineRemoteOn.text()
-        newConf["COM_REMOTEOFF"]    = self.ui.lineRemoteOff.text()
-        newConf["COM_POWER"]        = self.ui.linePower.text()
-        newConf["COM_REBOOT"]       = self.ui.lineReboot.text()
-        newConf["COM_POWEROFF"]     = self.ui.linePowerOff.text()
-        newConf["COM_CUSTOM"]       = self.ui.lineScript.text()
-        #newConf["COM_CUSTOMLOG"]    = self.ui.lineScriptLog.text()   -- TODO: ADD TO GUI
-        newConf["COM_SHELL"]        = self.ui.lineShell.text()
-        newConf["COM_ALARM"]        = self.ui.lineAlarm.text()
-        newConf["COM_RESTART"]      = self.ui.lineRestart.text()
+       newConf["COM_CHECK"]        = self.ui.lineCheck.text()
+       newConf["COM_LOG"]          = self.ui.lineLog.text()   
+       newConf["COM_LOCATION"]     = self.ui.lineLocation.text()
+       newConf["COM_TRACKON"]      = self.ui.lineTrackOn.text()
+       newConf["COM_TRACKOFF"]     = self.ui.lineTrackOff.text()
+       newConf["COM_CAMERA"]       = self.ui.lineCamera.text()
+       newConf["COM_CALL"]         = self.ui.lineCall.text()
+       newConf["COM_LOCK"]         = self.ui.lineLock.text()
+       newConf["COM_UNLOCK"]       = self.ui.lineUnlock.text()
+       newConf["COM_REMOTEON"]     = self.ui.lineRemoteOn.text()
+       newConf["COM_REMOTEOFF"]    = self.ui.lineRemoteOff.text()
+       newConf["COM_POWER"]        = self.ui.linePower.text()
+       newConf["COM_REBOOT"]       = self.ui.lineReboot.text()
+       newConf["COM_POWEROFF"]     = self.ui.linePowerOff.text()
+       newConf["COM_CUSTOM"]       = self.ui.lineScript.text()
+       newConf["COM_CUSTOMLOG"]    = self.ui.lineScriptLog.text()
+       newConf["COM_SHELL"]        = self.ui.lineShell.text()
+       newConf["COM_ALARM"]        = self.ui.lineAlarm.text()
+       newConf["COM_RESTART"]      = self.ui.lineRestart.text()
 
-        message =   'The command names have been successfully saved.'
-        if areFeedbackDetailsDesired:
-            message +=  '\n' \
-                    '\nCommon prefix: %s' % newConf['SMS_COMPREFIX'] + \
-                    '\nCommon suffix: %s' % newConf['SMS_COMSUFFIX']
-        EditConfigAndApply(newConf, message)
+       message =      'The command names have been successfully saved.'
+       if areFeedbackDetailsDesired:
+          message +=  '\n' \
+                      '\nCommon prefix: %s' % newConf['SMS_COMPREFIX'] + \
+                      '\nCommon suffix: %s' % newConf['SMS_COMSUFFIX']
+       EditConfigAndApply(newConf, message)
       
-## SSH Settings
-class smsconEditorSSH(QtGui.QMainWindow):
+class smsconEditorSSH(QtGui.QMainWindow):      ## SSH Settings
 
     def __init__(self, parent=None):
 
@@ -730,34 +749,45 @@ class smsconEditorSSH(QtGui.QMainWindow):
        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
        self.show()       
 
-       ##Connect the GUI Buttons with actions
-       QtCore.QObject.connect(self.ui.btnCreateConfigSSH, QtCore.SIGNAL('clicked()'), self.doCreateConfigSSH)
-
        ##Set UI values from editConfig values
        global editConf
-       self.ui.lineSSHUsername   .setText(editConf['REMOTEUSER'])
-       self.ui.lineSSHPassword   .setText(editConf['REMOTEPASSWORD'])
-       self.ui.lineSSHHost       .setText(editConf['REMOTEHOST'])
-       self.ui.lineSSHPort       .setText(editConf['REMOTEPORT'])
-    
+       self.ui.lineSSHUsername      .setText(editConf['REMOTEUSER'])
+       self.ui.lineSSHPassword      .setText(editConf['REMOTEPASSWORD'])
+       self.ui.lineSSHHost          .setText(editConf['REMOTEHOST'])
+       self.ui.spinBoxSSHPort       .setValue(int(editConf['REMOTEPORT']))
+       self.ui.lineSSHListenIP      .setText(editConf['REMOTELISTENIP'])
+       self.ui.spinBoxSSHListenPort .setValue(int(editConf['REMOTELISTENPORT']))
+       self.ui.spinBoxSSHLocalPort  .setValue(int(editConf['REMOTE2LOCALPORT']))
+
+    def closeEvent(self, event):
+       """Store settings on close."""
+       event.accept()
+
+       self.doCreateConfigSSH()
+       # self.doRestart();
+       
     def doCreateConfigSSH(self):
 
-        newConf = {}
-        newConf["REMOTEUSER"]       = self.ui.lineSSHUsername.text()
-        newConf["REMOTEPASSWORD"]   = self.ui.lineSSHPassword.text()
-        newConf["REMOTEHOST"]       = self.ui.lineSSHHost.text()
-        newConf["REMOTEPORT"]       = self.ui.lineSSHPort.text()
+       newConf = {}
+       newConf["REMOTEUSER"]       = self.ui.lineSSHUsername.text()
+       newConf["REMOTEPASSWORD"]   = self.ui.lineSSHPassword.text()
+       newConf["REMOTEHOST"]       = self.ui.lineSSHHost.text()
+       newConf["REMOTEPORT"]       = str(self.ui.spinBoxSSHPort.text())
+       newConf["REMOTELISTENIP"]   = self.ui.lineSSHListenIP.text()
+       newConf["REMOTELISTENPORT"] = str(self.ui.spinBoxSSHListenPort.text())
+       newConf["REMOTE2LOCALPORT"] = str(self.ui.spinBoxSSHLocalPort.text())
 
-        message =   'The reverse SSH settings have been successfully saved.'
-        if areFeedbackDetailsDesired:
-            message +=  '\n' \
-                    '\nUser name: %s'     %  newConf["REMOTEUSER"] + \
-                    '\nPassword: %s'        % (newConf["REMOTEPASSWORD"] and '******') + \
-                    '\nHost: %s - Port: %s' % (newConf["REMOTEHOST"], newConf["REMOTEPORT"])
-        EditConfigAndApply(newConf, message)
+       message =   'The reverse SSH settings have been successfully saved.'
+       if areFeedbackDetailsDesired:
+          message +=  '\n' \
+                      '\nUser name: %s'                  %  newConf["REMOTEUSER"] + \
+                      '\nPassword: %s'                   % (newConf["REMOTEPASSWORD"] and '******') + \
+                      '\nHost: %s - Port: %s'            % (newConf["REMOTEHOST"], newConf["REMOTEPORT"]) + \
+                      '\nListening to IP: %s - Port: %s' % (newConf["REMOTELISTENIP"], newConf["REMOTELISTENPORT"]) + \
+                      '\nForwarded Port: %s'             % (newConf["REMOTE2LOCALPORT"])
+       EditConfigAndApply(newConf, message)
 
-## Host Settings        
-class smsconEditorHost(QtGui.QMainWindow):
+class smsconEditorHost(QtGui.QMainWindow):     ## Host Settings
 
     def __init__(self, parent=None):
 
@@ -769,34 +799,37 @@ class smsconEditorHost(QtGui.QMainWindow):
        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
        self.show()       
 
-       ##Connect the GUI Buttons with actions
-       QtCore.QObject.connect(self.ui.btnCreateConfigHOST, QtCore.SIGNAL('clicked()'), self.doCreateConfigHOST)
-
        ##Set UI values from editConfig values
        global editConf
-       self.ui.lineHostAddress   .setText(editConf['CHECKHOST'])
-       self.ui.lineHostInterval  .setText(editConf['CHECKTIME'])
-       self.ui.checkBoxHostEnable.setChecked(configTrue(editConf['ENABLECHECKHOST']))
-       self.ui.lineHostAddress   .setEnabled(configTrue(editConf['ENABLECHECKHOST']))
-       self.ui.lineHostInterval  .setEnabled(configTrue(editConf['ENABLECHECKHOST']))
+       self.ui.lineHostAddress            .setText(editConf['CHECKHOST'])
+       self.ui.doubleSpinBoxHostInterval  .setValue(float(editConf['CHECKTIME']))
+       self.ui.checkBoxHostEnable         .setChecked(configTrue(editConf['ENABLECHECKHOST']))
+       self.ui.lineHostAddress            .setEnabled(configTrue(editConf['ENABLECHECKHOST']))
+       self.ui.doubleSpinBoxHostInterval  .setEnabled(configTrue(editConf['ENABLECHECKHOST']))
+
+    def closeEvent(self, event):
+       """Store settings on close."""
+       event.accept()
+
+       self.doCreateConfigHOST()
+       # self.doRestart();
        
     def doCreateConfigHOST(self):
 
-        newConf = {}
-        newConf["CHECKHOST"]        = self.ui.lineHostAddress.text()
-        newConf["CHECKTIME"]        = self.ui.lineHostInterval.text()
-        newConf["ENABLECHECKHOST"]  = configYesNo(self.ui.checkBoxHostEnable.isChecked())
+       newConf = {}
+       newConf["CHECKHOST"]        = self.ui.lineHostAddress.text()
+       newConf["CHECKTIME"]        = str(self.ui.doubleSpinBoxHostInterval.value())
+       newConf["ENABLECHECKHOST"]  = configYesNo(self.ui.checkBoxHostEnable.isChecked())
 
-        message =   'The remote host settings have been successfully saved.'
-        if areFeedbackDetailsDesired:
-            message +=  '\n' \
-                    '\nCheck enabled: %s' % newConf["ENABLECHECKHOST"] + \
-                    '\nURL to check: %s'    % newConf["CHECKHOST"] + \
-                    '\nCheck interval: %s'  % newConf["CHECKTIME"]
-        EditConfigAndApply(newConf, message)
+       message =   'The remote host settings have been successfully saved.'
+       if areFeedbackDetailsDesired:
+          message +=  '\n' \
+                      '\nCheck enabled: %s'   % newConf["ENABLECHECKHOST"] + \
+                      '\nURL to check: %s'    % newConf["CHECKHOST"] + \
+                      '\nCheck interval: %s'  % newConf["CHECKTIME"]
+       EditConfigAndApply(newConf, message)
 
-## TESTING
-class smsconEditorTest(QtGui.QMainWindow):
+class smsconEditorTest(QtGui.QMainWindow):     ## Testing of main functions for settings correctness.
 
     def __init__(self, parent=None):
 
@@ -873,9 +906,9 @@ class smsconEditorPasswordChange(QtGui.QDialog):
        NewPassword2 = self.ui.lineNewPassword2.text()
           
        if NewPassword != NewPassword2:
-             message = 'The two cases are not equal. Try again.'
-             showMessage(message)
-             return
+          message = 'The two cases are not equal. Try again.'
+          showMessage(message)
+          return
        message = 'Storing new password, plase wait...'
        showQuickMessage(message)
 
@@ -906,12 +939,11 @@ class smsconEditorAbout(QtGui.QMainWindow):
        self.parent = parent
        self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
-       self.setWindowTitle("SMSCON Editor About")
+       self.setWindowTitle("About SMSCON Editor")
 
        aboutScrollArea = QScrollArea(self)
        aboutScrollArea.setWidgetResizable(True)
        awidget = QWidget(aboutScrollArea)
-       #awidget.setMinimumSize(470,1100)
        awidget.setSizePolicy( QSizePolicy.Expanding, QSizePolicy.Expanding )
        aboutScrollArea.setSizePolicy( QSizePolicy.Expanding, QSizePolicy.Expanding )
        try:
@@ -929,15 +961,18 @@ class smsconEditorAbout(QtGui.QMainWindow):
        
        aboutLayout.addWidget(aboutIcon)
 
-       aboutLabel = QLabel('''<center><b>SMSCON Editor</b><br>version: %s
-       <br><br><b>The SMSCON Editor is a user interface to assist in the initialisation, configuration and testing of SMSCON.</b>
+       aboutLabel = QLabel('''<center>
+       <br><b>SMSCON Editor</b><br>version: %s
+       <br><br><b>The SMSCON Editor is a user interface to assist in the initialisation, 
+       configuration and testing of SMSCON.</b>
+       
        <br><br>Licenced under GPLv2
        <br>by <b>Christos Zamantzas</b> (Saturn)
-       <br><br>Besides editing options and modifing secret pass-phrases to control the phone, it allows to define email and sms contact, remote shell access (SSH) and a remote command server (HOST).
-       <br>
-       <br>For more details on usage and implications see the wiki pages (click buttons below).
-       <br>
-       <br><b>Thanks to:</b>
+       <br><br>Besides editing options and modifing secret pass-phrases to control the phone, 
+       it allows to define email and sms contact, remote shell access (SSH) and a remote command server (HOST).
+       <br><br>For more details on usage and implications see the wiki pages (click buttons below).
+       
+       <br><br><b>Thanks to:</b>
        <br><b>Frank Visser</b> (Digitalvoid) for 
        <br>for code to import, decrypt and encrypt the user variables from the configuration file.
        <br><br><b>Lutz Schwarz</b> (schwarz.ware@gmx.de)
